@@ -5,7 +5,7 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: qty-macros/src/quantity_attr_helper.rs $
-// $Revision: 2022-02-17T16:39:59+01:00 $
+// $Revision: 2022-05-04T21:33:17+02:00 $
 
 use convert_case::{Case, Casing};
 use proc_macro2::{Span, TokenStream};
@@ -61,7 +61,9 @@ pub(crate) fn parse_args(args: TokenStream) -> Option<DerivedAs> {
         "Unknown argument(s) given to attribute `quantity`.";
     const OPERATOR_ERROR: &str = "Binary expression with '*' or '/' expected.";
     const OPERAND_ERROR: &str = "Identifier expected.";
-    const ARGS_HELP: &str = "Use `#[quantity]`\n\
+    #[rustfmt::skip]
+    const ARGS_HELP: &str =
+        "Use `#[quantity]`\n\
          or  `#[quantity(<lhs_ident> * <rhs_ident>]`\n\
          or  `#[quantity(<lhs_ident> / <rhs_ident>]`.";
 
@@ -94,7 +96,11 @@ pub(crate) fn parse_args(args: TokenStream) -> Option<DerivedAs> {
 }
 
 pub(crate) fn parse_item(item: TokenStream) -> Item {
-    const ITEM_HELP: &str = "Use `#[quantity]\n  ...\nstruct <ident> {}`.";
+    #[rustfmt::skip]
+    const ITEM_HELP: &str =
+        "Use `#[quantity]\n\
+              ...\n\
+              struct <ident> {}`.";
 
     match syn::parse2::<Item>(item.clone()) {
         Ok(item) => item,
@@ -131,6 +137,7 @@ fn is_ref_unit_attr(attr: &syn::Attribute) -> bool {
 const ARGS_LIST_ERROR: &str =
     "A comma-separated list of 2 to 5 arguments expected.";
 
+#[rustfmt::skip]
 const UNIT_ATTR_HELP: &str =
     "Use `#[unit(<ident>, \"<symbol>\", <si_prefix>, <scale>, \"<doc>\")]`\n\
      or  `#[unit(<ident>, \"<symbol>\", <si_prefix>, <scale>)]`\n\
@@ -221,6 +228,7 @@ fn ref_unit_def_from_attr(ref_unit_attr: &syn::Attribute) -> UnitDef {
     const WRONG_NUMBER_OF_ARGS_ERROR: &str =
         "2, 3 or 4 comma-separated args expected.";
     const WRONG_TYPE_OF_ARG_ERROR: &str = "No scale expected for ref_unit.";
+    #[rustfmt::skip]
     const HELP: &str =
         "Use `#[ref_unit(<ident>, \"<symbol>\", <si_prefix>, \"<doc>\")]`\n\
          or  `#[ref_unit(<ident>, \"<symbol>\", <si_prefix>)]`\n\
@@ -250,6 +258,7 @@ fn unit_defs_with_scale_from_attrs(
     const WRONG_NUMBER_OF_ARGS_ERROR: &str =
         "3, 4 or 5 comma-separated args expected.";
     const NO_SCALE_ERROR: &str = "<scale> arg expected.";
+    #[rustfmt::skip]
     const HELP: &str =
         "Use `#[unit(<ident>, \"<symbol>\", <si_prefix>, <scale>, \"<doc>\")]`
          or  `#[unit(<ident>, \"<symbol>\", <si_prefix>, <scale>)]`\n\
@@ -278,7 +287,9 @@ fn unit_defs_without_scale_from_attrs(
 ) -> Vec<UnitDef> {
     const WRONG_NUMBER_OF_ARGS_ERROR: &str =
         "2 or 3 comma-separated args expected.";
-    const HELP: &str = "Use `#[unit(<ident>, \"<symbol>\", \"<doc>\")]`\n\
+    #[rustfmt::skip]
+    const HELP: &str =
+        "Use `#[unit(<ident>, \"<symbol>\", \"<doc>\")]`\n\
          or  `#[unit(<ident>, \"<symbol>\")]`.";
 
     let mut unit_defs: Vec<UnitDef> = vec![];
@@ -417,14 +428,12 @@ fn codegen_qty_single_unit(
         }
         impl Unit for #unit_enum_ident {
             type QuantityType = #qty_ident;
-            const REF_UNIT: Option<Self> = None;
             fn iter<'a>() -> core::slice::Iter<'a, Self> {
                 Self::VARIANTS.iter()
             }
             fn name(&self) -> &'static str { #unit_name }
             fn symbol(&self) -> &'static str { #unit_symbol }
             fn si_prefix(&self) -> Option<SIPrefix> { None }
-            fn scale(&self) -> Option<AmountT> { None }
         }
         #[derive(Copy, Clone, Debug)]
         pub struct #qty_ident {
@@ -448,6 +457,27 @@ fn codegen_qty_single_unit(
                 #unit_enum_ident::#unit_ident
             }
         }
+        impl Add<Self> for #qty_ident {
+            type Output = Self;
+            #[inline(always)]
+            fn add(self, rhs: Self) -> Self::Output {
+                Self::new(self.amount() + rhs.amount(), self.unit())
+            }
+        }
+        impl Sub<Self> for #qty_ident {
+            type Output = Self;
+            #[inline(always)]
+            fn sub(self, rhs: Self) -> Self::Output {
+                Self::new(self.amount() - rhs.amount(), self.unit())
+            }
+        }
+        impl Div<Self> for #qty_ident {
+            type Output = AmountT;
+            #[inline(always)]
+            fn div(self, rhs: Self) -> Self::Output {
+                self.amount() / rhs.amount()
+            }
+        }
     )
 }
 
@@ -588,14 +618,46 @@ fn codegen_qty_without_ref_unit(
         #code_unit_variants_array
         impl Unit for #unit_enum_ident {
             type QuantityType = #qty_ident;
-            const REF_UNIT: Option<Self> = None;
             fn iter<'a>() -> core::slice::Iter<'a, Self> {
                 Self::VARIANTS.iter()
             }
             #code_fn_name
             #code_fn_symbol
             fn si_prefix(&self) -> Option<SIPrefix> { None }
-            fn scale(&self) -> Option<AmountT> { None }
+        }
+        impl Eq for #qty_ident {}
+        impl PartialEq<Self> for #qty_ident {
+            #[inline(always)]
+            fn eq(&self, other: &Self) -> bool {
+                <Self as Quantity>::eq(self, other)
+            }
+        }
+        impl PartialOrd for #qty_ident {
+            #[inline(always)]
+            fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+                <Self as Quantity>::partial_cmp(self, other)
+            }
+        }
+        impl Add<Self> for #qty_ident {
+            type Output = Self;
+            #[inline(always)]
+            fn add(self, rhs: Self) -> Self::Output {
+                <Self as Quantity>::add(self, rhs)
+            }
+        }
+        impl Sub<Self> for #qty_ident {
+            type Output = Self;
+            #[inline(always)]
+            fn sub(self, rhs: Self) -> Self::Output {
+                <Self as Quantity>::sub(self, rhs)
+            }
+        }
+        impl Div<Self> for #qty_ident {
+            type Output = AmountT;
+            #[inline(always)]
+            fn div(self, rhs: Self) -> Self::Output {
+                <Self as Quantity>::div(self, rhs)
+            }
         }
     )
 }
@@ -637,7 +699,7 @@ fn codegen_fn_scale(
             let unit_scale: &syn::Lit = unit.scale.as_ref().unwrap();
             code = quote!(
                 #code
-                #unit_enum_ident::#unit_ident => Some(Amnt!(#unit_scale)),
+                #unit_enum_ident::#unit_ident => Amnt!(#unit_scale),
             )
         } else {
             // should not happen!
@@ -645,7 +707,7 @@ fn codegen_fn_scale(
         }
     }
     quote!(
-        fn scale(&self) -> Option<AmountT> {
+        fn scale(&self) -> AmountT {
             match self {
                 #code
             }
@@ -678,20 +740,55 @@ fn codegen_qty_with_ref_unit(
         #code_unit_variants_array
         impl Unit for #unit_enum_ident {
             type QuantityType = #qty_ident;
-            const REF_UNIT: Option<Self> =
-                Some(#unit_enum_ident::#ref_unit_ident);
             fn iter<'a>() -> core::slice::Iter<'a, Self> {
                 Self::VARIANTS.iter()
             }
             #code_fn_name
             #code_fn_symbol
             #code_fn_si_prefix
+        }
+        impl LinearScaledUnit for #unit_enum_ident {
+            const REF_UNIT: Self = #unit_enum_ident::#ref_unit_ident;
             #code_fn_scale
         }
         impl HasRefUnit for #qty_ident {
             const REF_UNIT: #unit_enum_ident =
                 #unit_enum_ident::#ref_unit_ident;
         }
+        impl Eq for #qty_ident {}
+        impl PartialEq<Self> for #qty_ident {
+            #[inline(always)]
+            fn eq(&self, other: &Self) -> bool {
+                <Self as HasRefUnit>::eq(self, other)
+            }
+        }
+        impl PartialOrd for #qty_ident {
+            #[inline(always)]
+            fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+                <Self as HasRefUnit>::partial_cmp(self, other)
+            }
+        }
+        impl Add<Self> for #qty_ident {
+            type Output = Self;
+            #[inline(always)]
+            fn add(self, rhs: Self) -> Self::Output {
+                <Self as HasRefUnit>::add(self, rhs)
+            }
+        }
+        impl Sub<Self> for #qty_ident {
+            type Output = Self;
+            #[inline(always)]
+            fn sub(self, rhs: Self) -> Self::Output {
+                <Self as HasRefUnit>::sub(self, rhs)
+            }
+        }
+        impl Div<Self> for #qty_ident {
+            type Output = AmountT;
+            #[inline(always)]
+            fn div(self, rhs: Self) -> Self::Output {
+                <Self as HasRefUnit>::div(self, rhs)
+            }
+        }
     )
 }
 
@@ -702,40 +799,6 @@ fn codegen_impl_std_traits(qty_ident: &syn::Ident) -> TokenStream {
                 <Self as Quantity>::fmt(self, f)
             }
         }
-        impl Eq for #qty_ident {}
-        impl PartialEq<Self> for #qty_ident {
-            #[inline(always)]
-            fn eq(&self, other: &Self) -> bool {
-                <Self as Quantity>::eq(self, other)
-            }
-        }
-        impl PartialOrd for #qty_ident {
-            #[inline(always)]
-            fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-                <Self as Quantity>::partial_cmp(self, other)
-            }
-        }
-        impl Add<Self> for #qty_ident {
-            type Output = Self;
-            #[inline(always)]
-            fn add(self, rhs: Self) -> Self::Output {
-                <Self as Quantity>::add(self, rhs)
-            }
-        }
-        impl Sub<Self> for #qty_ident {
-            type Output = Self;
-            #[inline(always)]
-            fn sub(self, rhs: Self) -> Self::Output {
-                <Self as Quantity>::sub(self, rhs)
-            }
-        }
-        impl Div<Self> for #qty_ident {
-            type Output = AmountT;
-            #[inline(always)]
-            fn div(self, rhs: Self) -> Self::Output {
-                <Self as Quantity>::div(self, rhs)
-            }
-        }
         impl Mul<#qty_ident> for AmountT {
             type Output = #qty_ident;
             #[inline(always)]
@@ -757,6 +820,27 @@ fn codegen_impl_std_traits(qty_ident: &syn::Ident) -> TokenStream {
                 Self::Output::new(self.amount() / rhs, self.unit())
             }
         }
+        impl<TQ: Quantity> Mul<Rate<TQ, #qty_ident>> for #qty_ident {
+            type Output = TQ;
+
+            fn mul(self, rhs: Rate<TQ, #qty_ident>) -> Self::Output {
+                let amnt: AmountT =
+                    (self / rhs.per_unit().as_qty()) / rhs.per_unit_multiple();
+                Self::Output::new(amnt * rhs.term_amount(), rhs.term_unit())
+            }
+        }
+        impl<PQ: Quantity> Div<Rate<#qty_ident, PQ>> for #qty_ident {
+            type Output = PQ;
+
+            fn div(self, rhs: Rate<#qty_ident, PQ>) -> Self::Output {
+                let amnt: AmountT =
+                    (self / rhs.term_unit().as_qty()) / rhs.term_amount();
+                Self::Output::new(
+                    amnt * rhs.per_unit_multiple(),
+                    rhs.per_unit()
+                )
+            }
+        }
     )
 }
 
@@ -774,7 +858,7 @@ fn codegen_impl_qty_mul_qty(
             type Output = #res_qty_ident;
             fn mul(self, rhs: #rhs_qty_ident) -> Self::Output {
                 let scale =
-                    self.unit().scale().unwrap() * rhs.unit().scale().unwrap();
+                    self.unit().scale() * rhs.unit().scale();
                 match Self::Output::unit_from_scale(scale) {
                     Some(unit) =>
                         Self::Output::new(self.amount() * rhs.amount(), unit),
@@ -850,7 +934,7 @@ fn codegen_impl_div_qties(
             type Output = #res_qty_ident;
             fn div(self, rhs: #rhs_qty_ident) -> Self::Output {
                 let scale =
-                    self.unit().scale().unwrap() / rhs.unit().scale().unwrap();
+                    self.unit().scale() / rhs.unit().scale();
                 match Self::Output::unit_from_scale(scale) {
                     Some(unit) =>
                         Self::Output::new(self.amount() / rhs.amount(), unit),
@@ -1098,6 +1182,7 @@ mod internal_fn_tests {
         let mut item = get_ast_basic_qty();
         let qty_def = analyze(&mut item);
         let code_unit_variants = codegen_unit_variants(&qty_def.units);
+        #[rustfmt::skip]
         assert_eq!(
             code_unit_variants.to_string(),
             "Pop , \
